#include <display_syscalls.h>
#include <keyboard_syscalls.h>
#include <keyboard.hpp>
#include "graphic_functions.h"
#include "console.h"

#define BUFFER_SIZE 1024

char inputBuffer[BUFFER_SIZE], *inputPtr;
int inputSize=0;
int cur=0;

void clear()
{
	inputSize = 0;
	cur = 0;
	Bdisp_AllClr_VRAM();
}

void putchar(char c)
{
	char str[2];
	str[0] = c;
	str[1] = 0;
	if(c == '\n') {
		cur = 21 * (cur/21 + 1);
	} else {
		locate_OS((cur%21)+1, (cur/21)+1);
		Print_OS(str, 0, 0);
		cur++;
	}
	if(cur >= 8*21) {
		vertical_scroll();
		cur -= 21;
		locate_OS(1, 8);
		PrintLine("                     ", 21);
		locate_OS(1, 8);
	}
	Bdisp_PutDisp_DD();
}

void print(char* str)
{
	for( ; *str ; str++)
		putchar(*str);
}

void print_int(int n)
{
	int i;
	if(n < 0) putchar('-'), n = -n;
	for(i=1 ; n/i ; i*=10);
	for(i/=10 ; i ; i/=10) putchar((n/i)%10+'0');
}

char getchar()
{
	if(inputSize) {
		inputSize--;
		inputPtr++;
	} else {
		unsigned int key = 0;
		inputPtr = inputBuffer;
		locate_OS((cur%21)+1, (cur/21)+1);
		Cursor_SetFlashOn(0);
		while(PRGM_GetKey() == 31);
		while(key != KEY_CTRL_EXE) {
			GetKey(&key);
			if(key == KEY_CTRL_DEL && inputSize) {
				cur--;
				putchar(' ');
				cur--;
				locate_OS((cur%21)+1, (cur/21)+1);
				inputSize--;
				inputPtr--;
			}
			if(key < 256 && inputSize<BUFFER_SIZE-1) {
				putchar(key);
				*inputPtr = key;
				inputPtr++;
				inputSize++;
			}
		}
		putchar('\n');
		*inputPtr = '\n';
		inputPtr = inputBuffer;
		Cursor_SetFlashOff();
		while(PRGM_GetKey() == 31);
	}
	return *inputPtr;
}

int PRGM_GetKey()
{
	unsigned char buffer[12];
	PRGM_GetKey_OS( buffer );
	return ( buffer[1] & 0x0F ) * 10 + ( ( buffer[2]  & 0xF0 )  >> 4 );
}
