#include <display_syscalls.h>
#include <HEAP_syscalls.h>
#include <rtc.h>
#include "graphic_functions.h"
#include "console.h"
#include "syscall.h"

#define BUFFER_SIZE 64
#define MAX_CALCS 9

void Sleep(int n)
{
	int i;
	for(i=1000*n ; i ; i--);
}

/**************************/
/** Networking constants **/
/**************************/
#define EOM 254 //end of message
#define BROADCAST 255

enum {IS_FREE, NOT_FREE, I_AM_HERE, START_ANIMATION, STOP_ANIMATION, BALL_TRANSFERT} Header;

void connect();
void disconnect();
void send(char recipient, void* data, int size);
void receive();

int network_id=0;

/*****************************/
/** Ball animation function **/
/*****************************/
#define MAX_NB_BALLS 1024
#define NB_BALLS 5

typedef struct {
	char type, x, y, dx, dy, width, height, sent;
} Ball;

Ball *balls[MAX_NB_BALLS];
int ball_animation_is_running;
int connected_calcs[MAX_CALCS+2];

void ball_add(Ball* ball)
{
	int i;
	for(i=0 ; balls[i] ; i++);
	if(i < MAX_NB_BALLS-1) balls[i] = ball;
}

void ball_del(int n)
{
	free(balls[n]);
	for( ; balls[n] ; n++) balls[n] = balls[n+1];
}

Ball* ball_new(int type, int x, int y, int dx, int dy)
{
	int size[] = {16, 16, 13, 8};
	Ball* ball = malloc(sizeof(Ball));
	ball->type = type;
	ball->x = x;
	ball->y = y;
	ball->dx = dx;
	ball->dy = dy;
	ball->width = ball->height = size[type];
	ball->sent = 0;
	return ball;
}

void ball_init()
{
	int i;
	srand(RTC_GetTicks());
	memset(balls, 0, MAX_NB_BALLS*sizeof(Ball*));
	for(i=0 ; i<NB_BALLS ; i++)
		ball_add(ball_new(rand()%4, rand()%112, rand()%48, (rand()%2+1)*((rand()%2)*2-1), (rand()%2)*2-1));
}

void ball_draw()
{
	char bmp[4][32] = {
		{7,128,30,120,61,252,123,254,123,254,247,255,207,255,63,255,255,255,255,254,255,253,127,250,127,230,63,156,31,120,6,224},
		{7,224,19,200,60,60,126,126,61,188,187,221,215,235,207,243,207,243,215,235,187,221,61,188,126,126,60,60,19,200,7,224},
		{15,128,63,224,117,112,106,176,213,88,234,184,213,88,234,184,213,88,106,176,117,112,63,224,15,128},
		{60,118,251,251,255,255,126,60}
	};
	int i;
	Bdisp_AllClr_VRAM();
	for(i=0 ; balls[i] ; i++)
		draw_bmp(bmp[balls[i]->type], balls[i]->x, balls[i]->y, balls[i]->width, balls[i]->height);
	Bdisp_PutDisp_DD();
}

void ball_move()
{
	int i;
	char buffer[sizeof(Ball)+1];
	buffer[0] = BALL_TRANSFERT;
	for(i=0 ; balls[i] ; i++) {
		balls[i]->x += balls[i]->dx;
		balls[i]->y += balls[i]->dy;
		if(balls[i]->y<=0) balls[i]->dy = abs(balls[i]->dy);
		if(balls[i]->y+balls[i]->height>=64) balls[i]->dy = -abs(balls[i]->dy);
		if(balls[i]->dx<0 && balls[i]->x < 0) {
			if(connected_calcs[network_id-1]) {
				if(!balls[i]->sent) {
					memcpy(buffer+1, balls[i], sizeof(Ball));
					send(network_id-1, buffer, sizeof(Ball)+1);
					balls[i]->sent = 1;
				}
			} else balls[i]->x += (balls[i]->dx *=-1);
		}
		if(balls[i]->dx>0 && balls[i]->x+balls[i]->width > 128) {
			if(connected_calcs[network_id+1]) {
				if(!balls[i]->sent) {
					memcpy(buffer+1, balls[i], sizeof(Ball));
					send(network_id+1, buffer, sizeof(Ball)+1);
					balls[i]->sent = 1;
				}
			} else balls[i]->x += (balls[i]->dx *=-1);
		}
		if(balls[i]->x+balls[i]->width<=0 || balls[i]->x>=128) ball_del(i);
	}
}

void ball_loop()
{
	int i, time=0;
	char buffer[2];
	ball_init();
	ball_animation_is_running = 1;

	//make the list of all connected calcs
	for(i=0 ; i<MAX_CALCS ; i++) connected_calcs[i] = 0;
	buffer[0] = I_AM_HERE;
	buffer[1] = network_id;
	send(BROADCAST, buffer, 2);
	Sleep(50);
	for(i=0 ; i<MAX_CALCS ; i++) receive();

	while(ball_animation_is_running) {
		if(PRGM_GetKey()==47) {
			char c = STOP_ANIMATION;
			send(BROADCAST, &c, 1);
			ball_animation_is_running = 0;
		}
		receive();
		if(RTC_GetTicks() - time > 5) {
			time = RTC_GetTicks();
			ball_move();
		}
		ball_draw();
	}
	while(balls[0]) ball_del(0);
	while(PRGM_GetKey()==47);
	clear();
	print("Waw, it was fun !\n");
}

/************************************************/
/** Networking functions.                      **/
/** Protocole : <recipient><sender><data><EOM> **/
/************************************************/
void send(char recipient, void* data, int size)
{
	Serial_WriteByte(network_id);
	Serial_WriteByte(recipient);
	Serial_WriteBytes((char*)data, size);
	Serial_WriteByte(EOM);
}

void receive()
{
	static unsigned char buffer[BUFFER_SIZE];
	static unsigned char size = 0;
	int i;
	if(network_id) {
		while(Serial_ReadByte(buffer+size) == 0) {
			if(buffer[size] == EOM) { //read received message
				if(buffer[1]==network_id || buffer[1]==BROADCAST) { //if i'm a recipient
					switch(buffer[2]) {
						case IS_FREE:
							if(buffer[3] == network_id) {
								buffer[0] = NOT_FREE;
								buffer[1] = network_id;
								send(BROADCAST, buffer, 2);
								print("Hey, it's me "); putchar(network_id+'0'); print("!\n");
							} break;
						case NOT_FREE:
							if(buffer[3] == network_id) {
								putchar(network_id+'0'); print(" is not free.\n");
								network_id = 0;
							} break;
						case I_AM_HERE: connected_calcs[buffer[3]] = 1; break;
						case START_ANIMATION: size=0; ball_loop(); break;
						case STOP_ANIMATION: ball_animation_is_running = 0; break;
						case BALL_TRANSFERT: ball_add(ball_new(buffer[3], (int)((char)buffer[4])+128*(buffer[0]-network_id), buffer[5], (char)buffer[6], (char)buffer[7])); break;
						default:
							print("From ");
							if(buffer[0]) putchar(buffer[0]+'0');
							else print("a guest");
							print(":");
							for(i=2 ; i<size ; i++) putchar(buffer[i]);
							print("\n");
					}
				}
				size = 0;
			} else size++; //message not ended
		}
	}
}

void connect()
{
	char conf[] = {0, 9, 0, 0, 0, 0};
	int i=0;
	while(Serial_Open(conf) != 0) {
		Serial_ClearReceiveBuffer();
		Serial_ClearTransmitBuffer();
		Serial_Close(1);
		if(++i == 3) {
			print("Unable to open serial port.\nPress [MENU] to quit.\n");
			while(1) getchar();
		}
	}
	Serial_ClearReceiveBuffer();
	Serial_ClearTransmitBuffer();
	print("Serial port open !\n");
}

void disconnect()
{
	send(BROADCAST, "I quit, ++", 10);
	Serial_ClearReceiveBuffer();
	while(Serial_Close(0) == 5);
}

/*******************/
/** main function **/
/*******************/
void main(void)
{
	char c=0, buffer[BUFFER_SIZE];

	print("Welcome on Flourish.\n\n");
	print("Open serial port :\n");
	connect();

	print("\nWhich number I am?\n(1-9)\n");
	do {
		do {
			if(c == '0') print("Hey, I'm not a zero!\n");
			else if(c) print("I want a digit dude.\n");
			c = getchar();
			while(getchar() != '\n');
		} while(c<'1' || c>'9');
		network_id = c-'0';
		c = 0;
		print("Number "); putchar(network_id+'0'); print("? Ok I try.\n");
		buffer[0] = IS_FREE;
		buffer[1] = network_id;
		send(BROADCAST, buffer, 2);
		Sleep(50);
		receive();
	} while(network_id == 0);
	print("Ok, I'm "); putchar(network_id+'0'); print(".\n");
	send(BROADCAST, "Hi girls!", 9);

	while(PRGM_GetKey()!=47) {
		receive();
		if(PRGM_GetKey()==31) {
			char recipient, size=0;
			print("Write to ");
			recipient = getchar();
			while(getchar() != '\n');
			print("Message:");
			while((c = getchar()) != '\n') buffer[size++] = c;
			send(recipient-'0', buffer, size);
		}
		if(PRGM_GetKey()==71) {
			buffer[0] = START_ANIMATION;
			send(BROADCAST, buffer, 1);
			ball_loop();
		}
		Sleep(30);
	}

	disconnect();
}
